import json
import os
import unittest
from unittest import mock

from src.georef_ar_py.diff import get_diff_object, DiffEntity


def get_mocked_response(url, entity, **kwargs):
    current_dir = os.path.dirname(os.path.abspath(__file__))
    entity_files = {
        'provincias': os.path.join(os.getcwd(), "tests/response_api/provincias.json"),
        'departamentos': os.path.join(os.getcwd(), "tests/response_api/departamentos.json"),
        'municipios': os.path.join(os.getcwd(), "tests/response_api/municipios.json"),
        'localidades-censales': os.path.join(os.getcwd(), "tests/response_api/localidades-censales.json"),
        'asentamientos': os.path.join(os.getcwd(), "tests/response_api/asentamientos.json"),
        'localidades': os.path.join(os.getcwd(), "tests/response_api/localidades.json"),
        'calles': os.path.join(os.getcwd(), "tests/response_api/calles.json"),
    }
    filename = os.path.join(current_dir, entity_files[entity])
    with open(filename, 'r') as f:
        data = json.loads(f.read())

    return data


def get_mocked_entity_number(url, entity, **kwargs):
    return get_mocked_response(url, entity)['total']


class DiffTestCase(unittest.TestCase):
    """
        Agrega tests para verificar la cantidad de entidades en cada capa y el correcto funcionamiento de la clase DiffEntity
    """

    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()

    def setUp(self) -> None:
        super().setUp()
        self.path_dir = os.getcwd()
        self.src_url = "http://52.23.185.155/georef/api/"
        self.target_url = "http://52.23.185.155/georef/api/"

    @mock.patch('src.georef_ar_py.diff.get_entity_number')
    @mock.patch.object(DiffEntity, "_get_response")
    def test_diff_provinces(self, mock_method, mock_entity_number):
        mock_entity_number.side_effect = get_mocked_entity_number
        mock_method.return_value = get_mocked_response(self.target_url, "provincias")

        diff_entity = get_diff_object(self.src_url, self.target_url, 'provincias')

        self.assertEqual(24, len(diff_entity.src_registers))
        self.assertEqual(24, len(diff_entity.target_registers))
        self.assertEqual({}, diff_entity._get_diff_as_dict())

    @mock.patch('src.georef_ar_py.diff.get_entity_number')
    @mock.patch.object(DiffEntity, "_get_response")
    def test_diff_departments(self, mock_method, mock_entity_number):
        mock_entity_number.side_effect = get_mocked_entity_number
        mock_method.return_value = get_mocked_response(self.target_url, "departamentos")

        diff_entity = get_diff_object(self.src_url, self.target_url, 'departamentos')

        self.assertEqual(len(diff_entity.src_registers), 10)
        self.assertEqual(len(diff_entity.target_registers), 10)
        self.assertEqual({}, diff_entity._get_diff_as_dict())

    @mock.patch('src.georef_ar_py.diff.get_entity_number')
    @mock.patch.object(DiffEntity, "_get_response")
    def test_diff_municipalities(self, mock_method, mock_entity_number):
        mock_entity_number.side_effect = get_mocked_entity_number
        mock_method.return_value = get_mocked_response(self.target_url, "municipios")

        diff_entity = get_diff_object(self.src_url, self.target_url, 'municipios')

        self.assertEqual(len(diff_entity.src_registers), 10)
        self.assertEqual(len(diff_entity.target_registers), 10)
        self.assertEqual({}, diff_entity._get_diff_as_dict())

    @mock.patch('src.georef_ar_py.diff.get_entity_number')
    @mock.patch.object(DiffEntity, "_get_response")
    def test_diff_census_localities(self, mock_method, mock_entity_number):
        mock_entity_number.side_effect = get_mocked_entity_number
        mock_method.return_value = get_mocked_response(self.target_url, "localidades-censales")

        diff_entity = get_diff_object(self.src_url, self.target_url, 'localidades-censales')

        self.assertEqual(len(diff_entity.src_registers), 10)
        self.assertEqual(len(diff_entity.target_registers), 10)
        self.assertEqual({}, diff_entity._get_diff_as_dict())

    @mock.patch('src.georef_ar_py.diff.get_entity_number')
    @mock.patch.object(DiffEntity, "_get_registers_by_region")
    def test_diff_settlements(self, mock_method, mock_entity_number):
        mock_entity_number.side_effect = get_mocked_entity_number
        mock_method.return_value = get_mocked_response(self.target_url, "asentamientos")['asentamientos']

        diff_entity = get_diff_object(self.src_url, self.target_url, 'asentamientos')

        self.assertEqual(len(diff_entity.src_registers), 10)
        self.assertEqual(len(diff_entity.target_registers), 10)
        self.assertEqual({}, diff_entity._get_diff_as_dict())

    @mock.patch('src.georef_ar_py.diff.get_entity_number')
    @mock.patch.object(DiffEntity, "_get_response")
    def test_diff_localities(self, mock_method, mock_entity_number):
        mock_entity_number.side_effect = get_mocked_entity_number
        mock_method.return_value = get_mocked_response(self.target_url, "localidades")

        diff_entity = get_diff_object(self.src_url, self.target_url, 'localidades')

        self.assertEqual(len(diff_entity.src_registers), 10)
        self.assertEqual(len(diff_entity.target_registers), 10)
        self.assertEqual({}, diff_entity._get_diff_as_dict())

    @mock.patch('src.georef_ar_py.diff.get_entity_number')
    @mock.patch.object(DiffEntity, "_get_registers_by_region")
    def test_diff_streets(self, mock_method, mock_entity_number):
        mock_entity_number.side_effect = get_mocked_entity_number
        mock_method.return_value = get_mocked_response(self.target_url, "calles")['calles']

        diff_entity = get_diff_object(self.src_url, self.target_url, 'calles')

        self.assertEqual(len(diff_entity.src_registers), 10)
        self.assertEqual(len(diff_entity.target_registers), 10)
        self.assertEqual({}, diff_entity._get_diff_as_dict())



import logging
from requests import RequestException

from .georequests import get_json
from .constants import ENTITIES, PROVINCES_DICT

log = logging.getLogger(__name__)


def _get_response(url, entity, **kwargs):
    return get_json(url, entity, **kwargs)


def get_entity_number(url, entity, **kwargs):
    log.debug(f"Consultando cantidad de registros en {entity}")
    if entity not in ENTITIES:
        raise NotImplemented(f"La entidad \"{entity}\" no se encuentra implementada.")
    kwargs.update({'campos': 'basico', 'max': 1})
    return _get_response(url, entity, **kwargs)['total']


def get_entities_number(url, **kwargs):
    entities = {}
    for entity in ENTITIES:
        try:
            entities.update({entity: get_entity_number(url, entity, **kwargs)})
        except RequestException:
            entities.update({entity: None})
    return entities


def get_departments_ids(url, state=None):
    params = {'campos': 'basico', 'orden': 'id', 'max': 5000}
    if state:
        params.update({'provincia': state})
    response = _get_response(url, 'departamentos', **params)
    return [department['id'] for department in response['departamentos']]


def _get_items(url, entity, entity_key=None, **kwargs):
    entity_key = entity_key or entity
    params = {'campos': 'completo', 'orden': 'id', 'max': kwargs.get('max', 5000)}
    response = _get_response(url, entity, **params)
    return response[entity_key]


def _get_items_by_region(url, entity, entity_key=None, **kwargs):

    entity_key = entity_key or entity
    limit = kwargs.get('max', 5000)

    params = {'campos': 'completo', 'orden': 'id', 'max': limit}

    item_list = []
    for state_id in PROVINCES_DICT.keys():
        max_items_province = get_entity_number(url, entity, provincia=state_id)
        if max_items_province <= limit:
            log.info(f"Consultando {entity}:provincia[{state_id}]")
            params.pop('departamento', None)
            params.update({'provincia': state_id})
            response = _get_response(url, entity, **params)
            item_list.extend(response[entity_key])
        else:
            for dep_id in get_departments_ids(url, state_id):
                log.info(f"Consultando {entity}:departamento[{dep_id}]")
                params.pop('provincia', None)
                params.update({'departamento': dep_id})
                response = _get_response(url, entity, **params)
                item_list.extend(response[entity_key])

    return item_list


def get_resume(url, **kwargs):

    resume = {}

    log.info(f"Consultando provincias")
    provinces_list = _get_items(url, 'provincias')
    resume.setdefault('provincias', {}).update({'total': len(provinces_list)})
    resume['provincias'].update({row['id']: {} for row in provinces_list})

    log.info(f"Consultando departamentos")
    departments_list = _get_items(url, 'departamentos')
    resume.setdefault('departamentos', {}).update({'total': len(departments_list)})
    for row in departments_list:
        resume['provincias'][row['provincia']['id']].setdefault('departamentos', 0)
        resume['provincias'][row['provincia']['id']]['departamentos'] += 1

    log.info(f"Consultando municipios")
    municipalities_list = _get_items(url, 'municipios')
    resume.setdefault('municipios', {}).update({'total': len(municipalities_list)})
    for row in municipalities_list:
        resume['provincias'][row['provincia']['id']].setdefault('municipios', 0)
        resume['provincias'][row['provincia']['id']]['municipios'] += 1

    log.info(f"Consultando localidades-censales")
    census_localities_list = _get_items(url, 'localidades-censales', entity_key='localidades_censales')
    resume.setdefault('localidades-censales', {}).update({'total': len(census_localities_list)})
    for row in census_localities_list:
        resume['provincias'][row['provincia']['id']].setdefault('localidades-censales', 0)
        resume['provincias'][row['provincia']['id']]['localidades-censales'] += 1

    log.info(f"Consultando asentamientos")
    settlements_list = _get_items_by_region(url, 'asentamientos')
    resume.setdefault('asentamientos', {}).update({'total': len(settlements_list)})
    for row in settlements_list:
        resume['provincias'][row['provincia']['id']].setdefault('asentamientos', 0)
        resume['provincias'][row['provincia']['id']]['asentamientos'] += 1

    log.info(f"Consultando localidades")
    localities_list = _get_items(url, 'localidades')
    resume.setdefault('localidades', {}).update({'total': len(localities_list)})
    for row in localities_list:
        resume['provincias'][row['provincia']['id']].setdefault('localidades', 0)
        resume['provincias'][row['provincia']['id']]['localidades'] += 1

    log.info(f"Consultando calles")
    streets_list = _get_items_by_region(url, 'calles')
    resume.setdefault('calles', {}).update({'total': len(streets_list)})
    for row in streets_list:
        resume['provincias'][row['provincia']['id']].setdefault('calles', 0)
        resume['provincias'][row['provincia']['id']]['calles'] += 1

    return resume



import json
import os
import unittest
from unittest import mock

import src.georef_ar_py
from src.georef_ar_py.similar import get_similar_bulk


def get_mocked_similar_san_juan():
    filename = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        os.path.join(os.getcwd(), "tests/response_api/similar_san_juan.json")
    )
    with open(filename, 'r') as f:
        return json.loads(f.read())


def get_mocked_similar_bulk():
    filename = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        os.path.join(os.getcwd(), "tests/response_api/similar_bulk.json")
    )
    with open(filename, 'r') as f:
        return json.loads(f.read())


class MyTestCase(unittest.TestCase):

    @mock.patch('src.georef_ar_py.similar.get_json')
    def test_get_similar(self, mock_get_json):
        mock_get_json.return_value = get_mocked_similar_san_juan()
        response = src.georef_ar_py.similar.get_similar(None, 'provincias', 'San Juan')
        self.assertEqual(response[0]['id'], '70')
        self.assertEqual(response[0]['nombre'], 'San Juan')

    @mock.patch('src.georef_ar_py.similar.get_json_post')
    def test_similar_bulk(self, mock_get_json):
        mock_get_json.return_value = get_mocked_similar_bulk()
        response = get_similar_bulk(None, "provincias", ["pxa", "sant fe"])
        self.assertEqual(len(response), 2)
        self.assertEqual(len(response[0]), 0)
        self.assertEqual(response[1]['id'], '82')
        self.assertEqual(response[1]['nombre'], 'Santa Fe')



import logging
from unittest import TestCase

import pytest as pytest
from requests import RequestException

from src.georef_ar_py import georequests
from src.georef_ar_py.georequests import get_json

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


class Test(TestCase):
    def test_get_json(self):
        with pytest.raises(RequestException):
            get_json(georequests.API_BASE_URL, 'paises')



import logging

import requests as requests

API_BASE_URL = "https://apis.datos.gob.ar/georef/api/"
TOKEN = None


def get_json(url, endpoint, **kwargs):
    """
        Obtiene una respuesta como json para la entidad consultada.
    :param url: URL de la API a consultar.
    :param endpoint: nombre de una de las capas.
    :param kwargs: parámetros de consulta.
    :return: Un diccionario con la respuesta obtenida
    """
    headers = {'Authorization': 'Bearer {}'.format(TOKEN)} if TOKEN and url == API_BASE_URL else None
    with requests.get("{}{}".format(url, endpoint), params=kwargs, headers=headers) as req:
        if req.status_code == 200:
            return req.json()
        raise requests.RequestException(req)


def get_json_post(url, endpoint, data, **kwargs):
    headers = kwargs.pop('headers', {})
    headers.update({"Content-Type": "application/json"})
    if TOKEN and url == API_BASE_URL:
        headers.update({'Authorization': 'Bearer {}'.format(TOKEN)})

    with requests.post("{}{}".format(url, endpoint), json=data, headers=headers) as req:
        print(req.request.url)
        if req.status_code == 200:
            return req.json()
        raise requests.RequestException(req)





import json
import logging
import re

import pandas as pd
from deepdiff import DeepDiff

from . import constants
from .georequests import get_json
from .info import get_entity_number, get_departments_ids

log = logging.getLogger(__name__)


class DiffEntity:
    def __init__(
            self, entity, src_url, target_url, entity_key=None, max_registries=None,
            ignore_order=True, significant_digits=1, exclude_paths=None, exclude_regex_paths=None, **kwargs
    ) -> None:
        super().__init__()
        self._entity = entity
        self._src_url = src_url
        self._target_url = target_url
        self._entity_key = entity_key or entity
        self._max_registries = max_registries
        self._ignore_order = ignore_order
        self._significant_digits = significant_digits
        self._exclude_paths = exclude_paths
        self._exclude_regex_paths = exclude_regex_paths
        self._src_registers = None
        self._target_registers = None
        self._diff_dict = None
        self._limit = kwargs.get('limit', 5000)
        self._max_src = None
        self._max_target = None

    def _get_response(self, url, **kwargs):
        return get_json(url, self._entity, **kwargs)

    @property
    def max_src(self):
        if not self._max_src:
            self._max_src = get_entity_number(self._src_url, self._entity)
        return self._max_src

    @property
    def max_target(self):
        if not self._max_target:
            self._max_target = get_entity_number(self._target_url, self._entity)
        return self._max_target

    def _get_max_entity_number(self):
        return max(self.max_src, self.max_target)

    def _get_registers(self, url, **kwargs):
        log.debug(f"Obteniendo registros de {url}")
        return {entity['id']: entity for entity in self._get_response(url, **kwargs)[self._entity_key]}

    def _get_registers_by_region(self, url, **kwargs):
        log.debug(f"Obteniendo registros de {url}")
        limit = kwargs.get('max', self._limit)

        params = {'campos': 'completo', 'orden': 'id', 'max': limit}

        item_dict = {}
        for state_id in constants.PROVINCES_DICT.keys():
            max_items_province = get_entity_number(url, self._entity, provincia=state_id)
            if max_items_province <= limit:
                log.info(f"Consultando {self._entity}:provincia[{state_id}]")
                params.pop('departamento', None)
                params.update({'provincia': state_id})
                item_dict.update(self._get_registers(url, **params))
            else:
                for dep_id in get_departments_ids(url, state_id):
                    log.info(f"Consultando {self._entity}:departamento[{dep_id}]")
                    params.pop('provincia', None)
                    params.update({'departamento': dep_id})
                    item_dict.update(self._get_registers(url, **params))

        return item_dict

    @property
    def src_registers(self):
        if not self._src_registers:
            if self.max_src < self._limit:
                self._src_registers = self._get_registers(self._src_url, campos="completo", orden="id", max=self._limit)
            else:
                self._src_registers = self._get_registers_by_region(self._src_url)
        return self._src_registers

    @property
    def target_registers(self):
        if not self._target_registers:
            if self.max_target < self._limit:
                self._target_registers = self._get_registers(self._target_url, campos="completo", orden="id", max=self._limit)
            else:
                self._target_registers = self._get_registers_by_region(self._target_url)
        return self._target_registers

    def _get_diff_as_dict(self):
        return DeepDiff(
            self.src_registers, self.target_registers,
            ignore_order=self._ignore_order, significant_digits=self._significant_digits,
            exclude_paths=self._exclude_paths, exclude_regex_paths=self._exclude_regex_paths
        )

    @property
    def diff_dict(self):
        if not self._diff_dict:
            self._diff_dict = self._get_diff_as_dict()
        return self._diff_dict

    def diff_as_json(self, filename):
        with open(filename, '+w') as f:
            json.dump(json.loads(self.diff_dict.to_json()), f)
            log.debug(f"Archivo generado: {filename}")

    def diff_as_csv(self, filename):

        diff_dict = {}

        def get_keys(string):
            return [str(k).replace("['", "").replace("']", "") for k in re.findall(r'\[.*?\]', string)]

        for row in self.diff_dict.get('dictionary_item_added', []):
            diff_dict.setdefault(get_keys(row)[0], {'added': True, 'removed': False})

        for row in self.diff_dict.get('dictionary_item_removed', []):
            diff_dict.setdefault(get_keys(row)[0], {'added': False, 'removed': True})

        for key, value in self.diff_dict.get('type_changes', {}).items():
            keys = get_keys(key)
            id = keys[0]
            head = ":".join(keys[1:])
            diff_dict.setdefault(id, {'added': False, 'removed': False}).update(
                {head + '(new)': value['new_value'], head + '(old)': value['old_value']}
            )

        for key, value in self.diff_dict.get('values_changed', {}).items():
            keys = get_keys(key)
            id = keys[0]
            head = ":".join(keys[1:])
            diff_dict.setdefault(id, {'added': False, 'removed': False}).update(
                {head + '(new)': value['new_value'], head + '(old)': value['old_value']}
            )

        if 'provincias' in self.diff_dict.keys():
            for pid in self.diff_dict['provincias'].keys():
                if 'values_changed' in self.diff_dict['provincias'][pid].keys():
                    for key, value in self.diff_dict['provincias'][pid]['values_changed'].items():
                        keys = get_keys(key)
                        id = keys[0]
                        head = ":".join(keys[1:])
                        diff_dict.setdefault(id, {'added': False, 'removed': False}).update(
                            {head + '(new)': value['new_value'], head + '(old)': value['old_value']}
                        )

        diff_df = pd.DataFrame.from_dict(diff_dict, orient='index')

        diff_df.to_csv(filename, index_label='id')

        log.debug(f"Archivo generado: {filename}")


def get_diff_object(src_url, target_url, entity):
    if entity in ['provincias', 'departamentos', 'municipios', 'asentamientos', 'localidades', 'calles']:
        return DiffEntity(entity, src_url, target_url)
    elif entity == 'localidades-censales':
        return DiffEntity(
            entity, src_url, target_url, entity_key='localidades_censales',
            exclude_regex_paths=None  # r"root\['\d+']\['centroide'\]"
        )
    raise NotImplemented(f'La capa {entity} no se encuentra implementada')



PROVINCES = 'provincias'
DEPARTMENTS = 'departamentos'
MUNICIPALITIES = 'municipios'
SETTLEMENTS = 'asentamientos'
LOCALITIES = 'localidades'
CENSUS_LOCALITIES = 'localidades_censales'
STREETS = 'calles'
INTERSECTIONS = 'intersecciones'
STREET_BLOCKS = 'cuadras'
SYNONYMS = 'sinonimos'
EXCLUDING_TERMS = 'terminos_excluyentes'

ENTITIES = [
    'provincias',
    'departamentos',
    'municipios',
    'localidades-censales',
    'asentamientos',
    'localidades',
    'calles'
]

PROVINCES_DICT = {
    '02': 'Ciudad Autónoma de Buenos Aires',
    '06': 'Buenos Aires',
    '10': 'Catamarca',
    '14': 'Córdoba',
    '18': 'Corrientes',
    '22': 'Chaco',
    '26': 'Chubut',
    '30': 'Entre Ríos',
    '34': 'Formosa',
    '38': 'Jujuy',
    '42': 'La Pampa',
    '46': 'La Rioja',
    '50': 'Mendoza',
    '54': 'Misiones',
    '58': 'Neuquén',
    '62': 'Río Negro',
    '66': 'Salta',
    '70': 'San Juan',
    '74': 'San Luis',
    '78': 'Santa Cruz',
    '82': 'Santa Fe',
    '86': 'Santiago del Estero',
    '90': 'Tucumán',
    '94': 'Tierra del Fuego, Antártida e Islas del Atlántico Sur'
}



from unittest import TestCase, mock

import tests.test_diff
from src.georef_ar_py import info
from src.georef_ar_py.georequests import API_BASE_URL


class Test(TestCase):

    @mock.patch('src.georef_ar_py.info.get_json')
    def test_get_entity_number(self, get_response_mock):

        get_response_mock.side_effect = tests.test_diff.get_mocked_response

        self.assertEqual(24, info.get_entity_number(API_BASE_URL, 'provincias'))

        self.assertEqual(529, info.get_entity_number(API_BASE_URL, 'departamentos'))

        self.assertEqual(2075, info.get_entity_number(API_BASE_URL, 'municipios'))

        self.assertEqual(4027, info.get_entity_number(API_BASE_URL, 'localidades-censales'))

        self.assertEqual(14673, info.get_entity_number(API_BASE_URL, 'asentamientos'))

        self.assertEqual(4037, info.get_entity_number(API_BASE_URL, 'localidades'))

        self.assertEqual(150054, info.get_entity_number(API_BASE_URL, 'calles'))

    @mock.patch('src.georef_ar_py.info.get_json')
    def test_get_resume(self, get_response_mock):

        get_response_mock.side_effect = tests.test_diff.get_mocked_response

        resume = info.get_resume(API_BASE_URL)

        self.assertTrue(all([key in resume.keys() for key in ['provincias', 'departamentos', 'municipios']]))



import urllib
from src.georef_ar_py.georequests import API_BASE_URL, get_json, get_json_post


def get_similar(url, endpoint, nombre, **kwargs):
    url = url or API_BASE_URL
    kwargs["nombre"] = nombre
    url = "{}{}?{}".format(url, endpoint, urllib.parse.urlencode(kwargs))
    return get_json(url, endpoint, **kwargs)[endpoint]


def get_similar_bulk(url, endpoint, nombres):
    """Normaliza una lista de nombres de alguna de las entidades geográficas."""

    url = url or API_BASE_URL
    data = {
        endpoint: [
            {"nombre": nombre, "max": 1} for nombre in nombres
        ]}
    results = get_json_post(url, endpoint, data)

    # convierte a una lista de "resultado más probable" o "vacío" cuando no hay
    parsed_results = [
        single_result[endpoint][0] if single_result[endpoint] else {}
        for single_result in results["resultados"]
    ]

    return parsed_results



from src.georef_ar_py.georequests import API_BASE_URL, get_json_post


def get_territorial_units(url, ubicaciones):
    """Pide las unidades territoriales que contienen a c/punto de una lista de coordenadas."""

    url = url or API_BASE_URL
    endpoint = "ubicacion"
    data = {
        "ubicaciones": [
            {"lat": ubicacion["lat"], "lon": ubicacion["lon"], "aplanar": True}
            for ubicacion in ubicaciones
        ]}

    results = get_json_post(url, endpoint, data)

    # convierte a una lista de "resultado más probable" o "vacío" cuando no hay
    parsed_results = [
        single_result[endpoint] if single_result[endpoint] else {}
        for single_result in results["resultados"]
    ]

    return parsed_results



import json
import os
import unittest
from unittest import mock

from src.georef_ar_py.territorial import get_territorial_units


def get_mocked_territorial():
    filename = os.path.join(
        os.path.dirname(os.path.abspath(__file__)),
        os.path.join(os.getcwd(), "tests/response_api/territorial.json")
    )
    with open(filename, 'r') as f:
        return json.loads(f.read())


class MyTestCase(unittest.TestCase):

    @mock.patch('src.georef_ar_py.territorial.get_json_post')
    def test_get_territorial_units(self, mock_get_json_post):
        mock_get_json_post.return_value = get_mocked_territorial()
        location1, location2 = get_territorial_units(None, [
            {"lat": -32.9477132, "lon": -60.6304658},
            {"lat": -34.6037389, "lon": -58.3815704}
        ])
        self.assertEqual('Santa Fe', location1['provincia_nombre'])
        self.assertEqual('Rosario', location1['departamento_nombre'])

        self.assertEqual(location2['provincia_nombre'], 'Ciudad Autónoma de Buenos Aires')
        self.assertEqual(location2['departamento_nombre'], 'Comuna 1')



import logging


def get_logger(level=logging.ERROR):

    logger = logging.getLogger(__name__)
    logger.setLevel(level)
    handler = logging.StreamHandler()
    formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(name)s - %(message)s")

    handler.setFormatter(formatter)

    logger.addHandler(handler)

    return logger



import json
import logging
import os

import click
from requests import RequestException

from . import georequests
from .constants import ENTITIES
from .diff import get_diff_object
from .georequests import API_BASE_URL
from .info import get_resume


def get_logger(level):
    logging.basicConfig(level=level, format="%(levelname)s - %(message)s")
    logger = logging.getLogger(__name__)
    return logger


@click.group()
def cli():
    pass


@cli.command()
@click.argument('url', type=str)
@click.option('--origin_url', required=False, type=str, show_default=True, default=API_BASE_URL)
@click.option('--token', required=False, type=str, show_default=True, default=None)
@click.option('--layer', required=False, type=click.Choice(ENTITIES, case_sensitive=True), show_default=True, default=None)
@click.option('--extension', required=False, type=click.Choice(['json', 'csv', 'both'], case_sensitive=True), show_default=True, default="both")
@click.option('--debug', is_flag=True, show_default=False)
def diff(*args, **kwargs):
    """
    geoarpy diff Commandline

    Calcula las diferencias de registros entre dos API's.
    Guarda el resultado en archivos de formato csv y/o json.

    url es el path a la API destino que se quiere comparar.
    """
    debug = kwargs.pop('debug')
    log = get_logger(logging.DEBUG if debug else logging.INFO)
    log.info("Comenzando la generación de diferencias...")

    src_url = kwargs.pop('origin_url')
    target_url = kwargs.pop('url')
    ext = kwargs.pop('extension')
    layer = kwargs.pop('layer')
    georequests.__dict__['TOKEN'] = kwargs.pop('token')

    path_dir = os.getcwd()

    if ext and ext not in ['json', 'csv', 'both']:
        raise NotImplemented(f'La extensión {ext} no está implementada.')

    entities = layer if layer else ENTITIES

    if isinstance(entities, str):
        entities = [layer]

    for layer in entities:
        log.info(f'Procesando la capa "{layer}"')

        try:
            diff_entity = get_diff_object(src_url, target_url, layer)
            if ext == 'both':
                diff_entity.diff_as_json(os.path.join(path_dir, f'diff_{layer}.json'))
                diff_entity.diff_as_csv(os.path.join(path_dir, f'diff_{layer}.csv'))
            elif ext == 'json':
                diff_entity.diff_as_json(os.path.join(path_dir, f'diff_{layer}.json'))
            else:
                diff_entity.diff_as_csv(os.path.join(path_dir, f'diff_{layer}.csv'))
        except RequestException as rqe:
            log.error(f'No se pudo procesar la petición {rqe.request}')


@cli.command()
@click.option('--url', required=False, type=str, show_default=True, default=API_BASE_URL)
@click.option('--token', required=False, type=str, show_default=True, default=None)
@click.option('--debug', is_flag=True, show_default=False)
def info(*args, **kwargs):
    """
    geoarpy info Commandline

    Obtienen un resumen de los registros en la API especificada

    url es el path a la API destino que se quiere consultar.
    """
    debug = kwargs.pop('debug')
    log = get_logger(logging.DEBUG if debug else logging.INFO)
    log.info("Comenzando la generación del resumen...")

    target_url = kwargs.pop('url')
    georequests.__dict__['TOKEN'] = kwargs.pop('token')

    resume = get_resume(target_url)

    filename = os.path.join(os.getcwd(), 'info.json')
    with open(filename, '+w') as f:
        json.dump(resume, f)
